@@ -1,17 +1,16 @@
-#!/usr/bin/env python2.6
+#!/usr/bin/env python2.7
 ################################################################################
 #
 #       This file is part of Gato (Graph Animation Toolbox) 
 #
-#   file:   GatoExport.py
-#   author: Alexander Schliep (alexander@schliep.org)
+#       file:   GatoExport.py
+#       author: Alexander Schliep (alexander@schlieplab.org)
 #
 #       Copyright (C) 2010, Alexander Schliep, Winfried Hochstaettler and 
-#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
+#       Copyright (C) 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
 #                                   
-#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de
+#       Contact: alexander@schlieplab.org
 #
-#       Information: http://gato.sf.net
 #
 #       This library is free software; you can redistribute it and/or
 #       modify it under the terms of the GNU Library General Public
@@ -30,8 +29,8 @@
 #
 #
 #
-#       This file is version $Revision: 353 $ 
-#                       from $Date: 2010-06-03 14:58:37 -0400 (Thu, 03 Jun 2010) $
+#       This file is version $Revision: 806 $ 
+#                       from $Date: 2020-12-08 15:13:58 +0100 (Tue, 08 Dec 2020) $
 #             last change by $Author: schliep $.
 #
 ################################################################################
@@ -73,6 +72,7 @@ svg_drop_shadow = '''
     <filter id="dropshadow" height="130%">
       <feGaussianBlur in="SourceAlpha" stdDeviation="1"/> <!-- stdDeviation is how much to blur -->
         <feOffset dx="2.5" dy="2.5" result="offsetblur"/> <!-- how much to offset -->
+
         <feMerge> 
           <feMergeNode/> <!-- this contains the offset blurred image -->
           <feMergeNode in="SourceGraphic"/> <!-- this contains the element that the filter is applied to -->
@@ -80,6 +80,8 @@ svg_drop_shadow = '''
     </filter>
 '''
 
+CHAPTER_NUM_TO_ABBREV = {2: 'Basics'}
+
 
 def tokenEater(type, token, (srow, scol), (erow, ecol), line):
     global line_count
@@ -215,7 +217,7 @@ def cmd_as_javascript(cmd, idPrefix=''):
     for arg in cmd.args:
         result.append(quote(arg))
     
-    # Special case for some animatin commands
+    # Special case for some animation commands
     if cmd.method.__name__ == 'SetAllVerticesColor' and 'vertices' in cmd.kwargs:
         for v in cmd.kwargs['vertices']:
             result.append(quote(v))
@@ -235,13 +237,15 @@ def collectAnimations(histories, prefixes):
     """ Given a list of animation histories (aka list of AnimationCommands)
         combine them, giving all targets of animation commands their history-
         specific prefix, sort them and return a list of JavaScripts arrays.
-    """ 
+    """
     mergedCmds = [cmd_as_javascript(cmd, prefixes[0]) for cmd in histories[0]]
     for i, h in enumerate(histories[1:]):
         mergedCmds += [cmd_as_javascript(cmd, prefixes[i+1]) for cmd in h]
     mergedCmds = sorted(mergedCmds, key=lambda cmd: cmd[0])
+
     # Replace absolute times by duration
     currentTime = mergedCmds[0][0]
+    start_idx = 0
     for i, cmd in enumerate(mergedCmds):
         duration = max(1,int(round((cmd[0] - currentTime) * 1000, 0)))
         currentTime = cmd[0]
@@ -250,13 +254,20 @@ def collectAnimations(histories, prefixes):
         for j in xrange(2, len(mergedCmds[i])):
             if 'Edge' in mergedCmds[i][1]:
                 mergedCmds[i][j] = change_id_format(mergedCmds[i][j])
+
+        # While we're looping, find the first ShowActive command.
+        # Prior to the first command there are just prolog commands which we can get rid of.
+        if cmd[1] == 'ShowActive' and cmd[2] == '"l_1"' and start_idx == 0:
+            start_idx = i
+
+    mergedCmds = mergedCmds[start_idx:]
     return ["Array(" + ", ".join(cmd) + ")" for cmd in mergedCmds]
 
 def get_edge_id(v, w, idPrefix):
     return idPrefix + '{}-{}'.format(v, w)
 
 
-def get_graph_as_svg_str_standalone(graphDisplay, x_add, y_add, file, idPrefix='', translate=None):
+def get_graph_as_svg_str_standalone(graphDisplay, x_add, y_add, file, idPrefix='', translate=None, skip_edge_info=False):
     ''' Returns a 3-tuple of (svg_string, width, height).  The SVG string returned
         is designed for standalone viewing.
     '''
@@ -387,10 +398,10 @@ def get_graph_as_svg_str_standalone(graphDisplay, x_add, y_add, file, idPrefix='
             size = r * 0.9
             offset = 0.33 * size
             col = 'black'
-            if text != "":
+            if text != "" and not skip_edge_info:
                 ret_strs.append('<text x="%s" y="%s" text-anchor="center" '\
                            'fill="%s" font-family="Helvetica" '\
-                           'font-size="%s" font-style="normal">%s</text>\n' % (ye+offset,col,size,text))
+                           'font-size="%s" font-style="normal">%s</text>\n' % (xe+offset,ye+offset,col,size,text))
 
     for v in graphDisplay.G.Vertices():
         x,y,r = graphDisplay.VertexPositionAndRadius(v)
@@ -426,15 +437,19 @@ def get_graph_as_svg_str_standalone(graphDisplay, x_add, y_add, file, idPrefix='
     if translate:
         ret_strs.append('</g>')
 
-    width = max_x-min_x+vertex_radius*2
-    height = max_y-min_y+vertex_radius*2
-    return '\n'.join(ret_strs), width, height
+    if all([min_x, max_x, vertex_radius]):
+        width = max_x-min_x+vertex_radius*2
+        height = max_y-min_y+vertex_radius*2
+        return '\n'.join(ret_strs), width, height
+    else:
+        return '', 0, 0
 
 
 
 def get_graph_as_svg_str_for_animation(graphDisplay, x_add, y_add, file, idPrefix=''):
     ''' Returns an svg string.  The SVG string returned 
-        is designed for use with javascript with WebGato
+        is designed for use with javascript with WebGato.
+        x_add and y_add are offsets for transforming coordinate system to (0, 0) origin
     '''
     ret_strs = []
 
@@ -599,8 +614,7 @@ def format_init_edge_infos(info_dict, idPrefix):
     if not info_dict:
         return 'null';
     str_bits = ['{'] # List of strings to return joined at the end(faster than concatenation)
-    for tup, info in info_dict.iteritems():
-        v, w = tup
+    for (v, w), info in info_dict.iteritems():
         edge_id = get_edge_id(v, w, idPrefix)
         assignment = '"{}": "{}",'.format(edge_id, info)
         str_bits.append(assignment)
@@ -670,26 +684,55 @@ def format_animation(animation):
     else:
         ret_str = 'var anim_array = Array(' + ',\n'.join(animation[:32000]) + ');\n'
         for chunk in chunker(animation[32000:], 32000):
-            ret_str += 'anim_array.concat(Array(' + ','.join(chunk) + '));\n'
+            ret_str += 'anim_array = anim_array.concat(Array(' + ','.join(chunk) + '));\n'
         return ret_str
 
 def construct_title(fileName):
-    sp = fileName.split('/')[1].split('--')
-    algorithm = sp[0]
-    graph = sp[1].split('.')[0]
-    return 'Gato -- %s algorithm on %s graph' % (algorithm, graph)
+    basename = os.path.splitext(os.path.basename(fileName))
+    algorithm, graph = basename[0].split('--')
+    result = 'Gato -- %s algorithm on %s graph' % (algorithm, graph)
+    print result
+    return result
 
 def construct_animation_name(fileName):
-    sp = fileName.split('/')[1].split('--')
-    algorithm = sp[0]
-    graph = sp[1].split('.')[0]
-    return '%s algorithm on %s graph' % (algorithm, graph)
+    basename = os.path.splitext(os.path.basename(fileName))
+    algorithm, graph = basename[0].split('--')
+    result = '%s on %s graph' % (algorithm, graph)
+    print result
+    return result
+    
+
+def get_start_coordinate_diff(graphDisplay, secondaryGraphDisplay=None):
+    start_g1_x_add, start_g1_y_add, start_g1_has_elements = compute_coord_changes(graphDisplay)
+    start_g2_x_add, start_g2_y_add, start_g2_has_elements = 0, 0, False
+    if secondaryGraphDisplay:
+        start_g2_x_add, start_g2_y_add, start_g2_has_elements = compute_coord_changes(secondaryGraphDisplay)
+        
+    return {
+        'g1_x_add': start_g1_x_add,
+        'g1_y_add': start_g1_y_add,
+        'g1_has_elements': start_g1_has_elements,
+        'g2_x_add': start_g2_x_add,
+        'g2_y_add': start_g2_y_add,
+        'g2_has_elements': start_g2_has_elements
+    }
+
+def format_bubbles(bubbles):
+    if not bubbles:
+        return '{}'
+    bubble_str = '{\n'
+    for bubble_id, (offset_value, color) in bubbles.iteritems():
+        bubble_str += '"%s"' % bubble_id
+        bubble_str += ': ["' + offset_value + '", "' + color + '"],\n'
+    bubble_str += '}'
+    return bubble_str
 
 def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=None, 
-    secondaryGraphDisplayAnimationHistory=None, showAnimation=False, 
-    init_edge_infos=None, init_vertex_infos=None, init_graph_infos=None,
-    write_to_png=False, chapter_number=None, algo_div=None):
-    """ Export either the current graphs or the complete animation
+              secondaryGraphDisplayAnimationHistory=None, showAnimation=False, 
+              init_edge_infos=None, init_vertex_infos=None, init_graph_infos=None,
+              write_to_png=False, chapter_number=None, algo_div=None, chapter_name=None, start_graph_coord_diff=None,
+              restart_algorithm=True, no_info_file=False):
+    """ **** OBSOLETE ***** Export either the current graphs or the complete animation
         (showAnimation=True) to the file fileName.
 
         showAnimation: We don't do anything if this is False
@@ -727,20 +770,32 @@ def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=
             print "graphDisplay: ", graphDisplay
             return
 
-    # Reload the graph and execute prolog so we can save the initial state to SVG
-    algorithm.Start(prologOnly=True)
+    
 
-    start_g1_x_add, start_g1_y_add, start_g1_has_elements = compute_coord_changes(graphDisplay)
+    # Reload the graph and execute prolog so we can save the initial state to SVG
+    # if showAnimation and not write_to_png:
+    if restart_algorithm:
+        algorithm.Start(prologOnly=True)
+
+    # If we have coord diffs passed in, then use those, otherwise recompute
+    if start_graph_coord_diff:
+        start_g1_x_add, start_g2_x_add = start_graph_coord_diff['g1_x_add'], start_graph_coord_diff['g2_x_add']
+        start_g1_y_add, start_g2_y_add = start_graph_coord_diff['g1_y_add'], start_graph_coord_diff['g2_y_add']
+        start_g1_has_elements, start_g2_has_elements = start_graph_coord_diff['g1_has_elements'], start_graph_coord_diff['g2_has_elements']
+    else:
+        start_g1_x_add, start_g1_y_add, start_g1_has_elements = compute_coord_changes(graphDisplay)
+        start_g2_x_add, start_g2_y_add, g2_x_add, g2_y_add = 0, 0, 0, 0
+        if secondaryGraphDisplay:
+            start_g2_x_add, start_g2_y_add, start_g2_has_elements = compute_coord_changes(secondaryGraphDisplay)
+    
+    # Find final g1 and g2 x and y coord differences
     if start_g1_has_elements and end_g1_has_elements:
         g1_x_add, g1_y_add = min(start_g1_x_add, end_g1_x_add), min(start_g1_y_add, end_g1_y_add)
     elif start_g1_has_elements:
         g1_x_add, g1_y_add = start_g1_x_add, start_g1_y_add
     else:
         g1_x_add, g1_y_add = end_g1_x_add, end_g1_y_add
-
-    start_g2_x_add, start_g2_y_add, g2_x_add, g2_y_add = 0, 0, 0, 0
     if secondaryGraphDisplay:
-        start_g2_x_add, start_g2_y_add, start_g2_has_elements = compute_coord_changes(secondaryGraphDisplay)
         if start_g2_has_elements and end_g2_has_elements:
             g2_y_add, g2_x_add = min(start_g2_y_add, end_g2_y_add), min(start_g2_x_add, end_g2_x_add)
         elif start_g2_has_elements:
@@ -748,6 +803,7 @@ def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=
         else:
             g2_y_add, g2_x_add = end_g2_y_add, end_g2_x_add
 
+
     if showAnimation and not write_to_png:
         # Build the SVG graph string
         graph_strs = []
@@ -769,13 +825,21 @@ def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=
         tokenize.tokenize(StringIO.StringIO(source.replace('\\', '\\\\')).readline, tokenEater)
         algowin.CommitStop()
 
+        # 
+        if no_info_file:
+            info_file = ""
+        else:
+            info_file = 'infos/' + fileName[fileName.rindex('/') + 1:]
+        
         # Merge the animation into the HTML
         str_vars = {
             'title': construct_title(fileName),
             'animation_name': construct_animation_name(fileName),
             'chapter_number': chapter_number or 0,
             'algo_div': algo_div or '',
-            'info_file': 'infos/' + fileName[fileName.rindex('/') + 1:], 
+            'chapter_name': CHAPTER_NUM_TO_ABBREV.get(int(chapter_number or 0)) or '',
+            'info_file': info_file,
+            'this_url': fileName[fileName.rindex('/') + 1:],
             'animation': format_animation(animation),
             'graph_str': '\n'.join(graph_strs), 
             'algo_str': '<g id="codelines" style="visibility: hidden">' + ''.join(algo_lines) + "</g>",
@@ -789,6 +853,10 @@ def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=
             'g2_init_graph_info': '"%s"' % init_graph_infos[1] if init_graph_infos and init_graph_infos[1] else '""',
             'g1_init_vertex_info': format_init_vertex_infos(init_vertex_infos[0], id_prefixes[0]) if init_vertex_infos else 'null',
             'g2_init_vertex_info': format_init_vertex_infos(init_vertex_infos[1], id_prefixes[1]) if init_vertex_infos else 'null',
+            'g1_init_bubbles': format_bubbles(graphDisplay.bubbles),
+            'g2_init_bubbles': format_bubbles(secondaryGraphDisplay.bubbles if secondaryGraphDisplay else None),
+            'g1_init_moats': format_bubbles(graphDisplay.moats),
+            'g2_init_moats': format_bubbles(secondaryGraphDisplay.moats if secondaryGraphDisplay else None),
         }
         file.write(animationhead % str_vars)
         file.close()
@@ -832,22 +900,22 @@ def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=
 
         edge_padding = 15
         g1_svg_body_str, g1_width, g1_height = get_graph_as_svg_str_standalone(graphDisplay, g1_x_add, 
-            g1_y_add, file, idPrefix=id_prefixes[0], translate=(edge_padding, edge_padding))
+            g1_y_add, file, idPrefix=id_prefixes[0], translate=(edge_padding, edge_padding), skip_edge_info=True)
         g2_svg_body_str, g2_width, g2_height = '', 0, 0
         g2_y_padding = 0
         if secondaryGraphDisplay:
             g2_y_padding = 10
             g2_svg_body_str, g2_width, g2_height = get_graph_as_svg_str_standalone(secondaryGraphDisplay, g2_x_add, 
-                g2_y_add, file, idPrefix=id_prefixes[1], translate=(edge_padding,g1_height+g2_y_padding+edge_padding))
+                g2_y_add, file, idPrefix=id_prefixes[1], translate=(edge_padding,g1_height+g2_y_padding+edge_padding), skip_edge_info=True)
 
         width = int(max(g1_width, g2_width))+edge_padding*2
         height = int(g1_height+g2_height+g2_y_padding)+edge_padding*2 
         scale = min(min(200.0/width, 1.0), min(200.0/height, 1.0))
         
         # Put a border between the graphs
-        if secondaryGraphDisplay:
+        if secondaryGraphDisplay and g2_svg_body_str:
             y = g1_height + g2_y_padding/2 + edge_padding
-            g2_svg_body_str += '<line x1="0" x2="%d" y1="%d" y2="%d" stroke="#000" stroke-width="3.0" />' % (width, y, y)
+            g2_svg_body_str += '<line x1="0" x2="%d" y1="%d" y2="%d" stroke="#aaa" stroke-width="1.0" />' % (width, y, y)
         svg_str = '\n'.join(['<svg width="%d" height="%d">' % (width, height), svg_drop_shadow, g1_svg_body_str, g2_svg_body_str, '</svg>'])
 
         img = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(ceil(width*scale)), int(ceil(height*scale)))
@@ -859,3 +927,254 @@ def ExportSVG(fileName, algowin, algorithm, graphDisplay, secondaryGraphDisplay=
         img.write_to_png(fileName)
         return {'width': width*scale, 'height': height*scale}
 
+
+
+
+
+def ComputeGraphPadding(graphDisplay, secondaryGraphDisplay, start_graph_coord_diff = None, endPadding = None):
+
+    if endPadding:
+        end_g1_x_add, end_g1_y_add, end_g1_has_elements, end_g2_x_add, end_g2_y_add, end_g2_has_elements = endPadding
+    else:
+        # Figure out how much we want to pull the graph to the left and top before we reset the graph
+        end_g1_x_add, end_g1_y_add, end_g1_has_elements = compute_coord_changes(graphDisplay)
+        end_g2_x_add, end_g2_y_add = 0, 0
+        if secondaryGraphDisplay:
+            end_g2_x_add, end_g2_y_add, end_g2_has_elements = compute_coord_changes(secondaryGraphDisplay)
+        else:
+            end_g2_x_add, end_g2_y_add, end_g2_has_elements = None, None, None
+       
+        
+    g2_y_add, g2_x_add = None, None
+    # If we have coord diffs passed in, then use those, otherwise recompute
+    if start_graph_coord_diff:
+        start_g1_x_add, start_g2_x_add = start_graph_coord_diff['g1_x_add'], start_graph_coord_diff['g2_x_add']
+        start_g1_y_add, start_g2_y_add = start_graph_coord_diff['g1_y_add'], start_graph_coord_diff['g2_y_add']
+        start_g1_has_elements, start_g2_has_elements = start_graph_coord_diff['g1_has_elements'], start_graph_coord_diff['g2_has_elements']
+    else:
+        start_g1_x_add, start_g1_y_add, start_g1_has_elements = compute_coord_changes(graphDisplay)
+        start_g2_x_add, start_g2_y_add, g2_x_add, g2_y_add = 0, 0, 0, 0
+        if secondaryGraphDisplay:
+            start_g2_x_add, start_g2_y_add, start_g2_has_elements = compute_coord_changes(secondaryGraphDisplay)
+    
+    # Find final g1 and g2 x and y coord differences
+    if start_g1_has_elements and end_g1_has_elements:
+        g1_x_add, g1_y_add = min(start_g1_x_add, end_g1_x_add), min(start_g1_y_add, end_g1_y_add)
+    elif start_g1_has_elements:
+        g1_x_add, g1_y_add = start_g1_x_add, start_g1_y_add
+    else:
+        g1_x_add, g1_y_add = end_g1_x_add, end_g1_y_add
+
+    if secondaryGraphDisplay:
+        if start_g2_has_elements and end_g2_has_elements:
+            g2_y_add, g2_x_add = min(start_g2_y_add, end_g2_y_add), min(start_g2_x_add, end_g2_x_add)
+        elif start_g2_has_elements:
+            g2_y_add, g2_x_add = start_g2_y_add, start_g2_x_add
+        else:
+            g2_y_add, g2_x_add = end_g2_y_add, end_g2_x_add
+
+    return g1_x_add, g1_y_add, g2_x_add, g2_y_add
+
+    
+
+def SVGForExport(algorithm, graphDisplay, secondaryGraphDisplay=None, initialState=False,
+                 edge_padding=5, skip_edge_info=False, scale_both_coordinates=False):
+    """ Compute the SVG as a string for exporting to SVG or a PNG """
+    if initialState:
+        algorithm.Start(prologOnly=True)
+
+    g1_x_add, g1_y_add, g2_x_add, g2_y_add = ComputeGraphPadding(graphDisplay, secondaryGraphDisplay)
+    g1_svg_body_str, g1_width, g1_height = get_graph_as_svg_str_standalone(graphDisplay,
+                                                                           g1_x_add,  g1_y_add,
+                                                                           file, idPrefix=id_prefixes[0],
+                                                                           translate=(edge_padding, edge_padding),
+                                                                           skip_edge_info=skip_edge_info)
+    g2_svg_body_str, g2_width, g2_height = '', 0, 0
+    g2_y_padding = 0
+    if secondaryGraphDisplay:
+        g2_y_padding = 10
+        g2_svg_body_str, g2_width, g2_height = get_graph_as_svg_str_standalone(secondaryGraphDisplay,
+                                                                               g2_x_add, g2_y_add,
+                                                                               file, idPrefix=id_prefixes[1],
+                                                                               translate=(edge_padding,g1_height+g2_y_padding+edge_padding),
+                                                                               skip_edge_info=skip_edge_info)
+
+    width = int(max(g1_width, g2_width))+edge_padding*2
+    height = int(g1_height+g2_height+g2_y_padding)+edge_padding*2
+    if scale_both_coordinates:
+        scale = min(min(200.0/width, 1.0), min(200.0/height, 1.0))
+    else:
+        scale = min(200.0/width, 1.0)
+        
+    # Put a border between the graphs
+    if secondaryGraphDisplay:
+        y = g1_height + g2_y_padding/2 + edge_padding
+        g2_svg_body_str += '<line x1="0" x2="%d" y1="%d" y2="%d" stroke="#000" stroke-width="3.0" />' % (width, y, y)
+    svg_str = '\n'.join(['<svg width="%d" height="%d">' % (width, height), svg_drop_shadow, g1_svg_body_str, g2_svg_body_str, '</svg>'])
+    return svg_str
+    
+
+
+def ExportGraphToSVG(svgFileName, algorithm, graphDisplay, secondaryGraphDisplay = None, initialState = False):
+    """ Export one or both graphs, if secondaryGraphDisplay is not none, currently displayed as a SVG.  
+        If initialState is True, the algorithm will be restarted to show graphs in their initial state.
+    """
+
+    svg_str = SVGForExport(algorithm, graphDisplay, secondaryGraphDisplay,
+                           initialState=initialState)
+    
+    file = open(svgFileName, 'w')
+    file.write(svg_str)
+    file.close()
+
+
+def ExportGraphToPNG(pngFileName, algorithm, graphDisplay, secondaryGraphDisplay = None, initialState = False):
+    """ Export one or both graphs, if secondaryGraphDisplay is not none, currently displayed as a PNG.  
+        If initialState is True, the algorithm will be restarted to show graphs in their initial state.
+    """
+
+    svg_str = SVGForExport(algorithm, graphDisplay, secondaryGraphDisplay,
+                           initialState=initialState,
+                           edge_padding=15,
+                           skip_edge_info=True,
+                           scale_both_coordinates=True)
+    
+    '''
+        The commands on Ubuntu to get these libraries installed:
+        sudo apt-get install libcairo2-dev librsvg2-dev python-rsvg
+    '''
+    import cairo
+    import rsvg
+    img = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(ceil(width*scale)), int(ceil(height*scale)))
+
+    ctx = cairo.Context(img)
+    ctx.scale(scale, scale)
+    handler= rsvg.Handle(None, svg_str)
+    handler.render_cairo(ctx)
+    img.write_to_png(fileName)
+    return {'width': width*scale, 'height': height*scale}
+ 
+    
+   
+def ExportAnimationAsHTML(fileName, algowin, algorithm, graphDisplay,
+                          secondaryGraphDisplay=None, secondaryGraphDisplayAnimationHistory=None, 
+                          init_edge_infos=None, init_vertex_infos=None, init_graph_infos=None,
+                          chapter_number=None, algo_div=None, chapter_name=None, start_graph_coord_diff=None,
+                          restart_algorithm=True, no_info_file=False,
+                          htmlTitle=None, animationName=None):
+    """ Export the complete animation to the HTML file fileName.
+
+        showAnimation: We don't do anything if this is False
+        init_edge_infos: list of dictionaries with keys that are tuples of form (v1, v2) where v1 and v2 are the vertices
+            that an edge connects.  The values keys point to are the initial edge infos of that edge.
+    """
+    global algo_lines
+
+    algo_lines = []
+    file = open(fileName, 'w')
+
+    # Figure out how much we want to pull the graph to the left and top before we reset the graph
+    end_g1_x_add, end_g1_y_add, end_g1_has_elements = compute_coord_changes(graphDisplay)
+    end_g2_x_add, end_g2_y_add = 0, 0
+    if secondaryGraphDisplay:
+        end_g2_x_add, end_g2_y_add, end_g2_has_elements = compute_coord_changes(secondaryGraphDisplay)
+    else:
+        end_g2_x_add, end_g2_y_add, end_g2_has_elements = None, None, None
+       
+        
+    try:
+        if secondaryGraphDisplayAnimationHistory:
+            animation = collectAnimations([algorithm.animation_history.getHistoryOne(),
+                                           secondaryGraphDisplayAnimationHistory.getHistoryTwo(),
+                                           algowin.codeLineHistory],
+                                          ['g1_', 'g2_', 'l_'])
+        else:
+            animation = collectAnimations([algorithm.animation_history.getHistoryOne(),
+                                           algowin.codeLineHistory],
+                                          ['g1_', 'l_'])
+    except IndexError as e:
+        print "Error:"
+        print e
+        print "Filename: ", fileName
+        print "Algowin: ", algowin
+        print "Algorithm: ", algorithm
+        print "graphDisplay: ", graphDisplay
+        return
+
+
+    # Reload the graph and execute prolog so we can save the initial state to SVG
+    # if showAnimation and not write_to_png:
+    if restart_algorithm:
+        algorithm.Start(prologOnly=True)
+
+
+    g1_x_add, g1_y_add, g2_x_add, g2_y_add = ComputeGraphPadding(graphDisplay, secondaryGraphDisplay,
+                                                                 endPadding = (end_g1_x_add, end_g1_y_add, end_g1_has_elements,
+                                                                               end_g2_x_add, end_g2_y_add, end_g2_has_elements)
+                                                                 )
+
+    # Build the SVG graph string
+    graph_strs = []
+    graph_type = "undirected" if graphDisplay.G.directed == 0 else "directed"
+    graph_strs.append('<g id="g1" type="%s" style="visibility: hidden">\n' % (graph_type))
+
+    g1_str = get_graph_as_svg_str_for_animation(graphDisplay, g1_x_add, g1_y_add, file, idPrefix=id_prefixes[0])
+    graph_strs.append(g1_str)
+    graph_strs.append('</g>\n')
+    if secondaryGraphDisplay:
+        graph_type = "undirected" if secondaryGraphDisplay.G.directed == 0 else "directed"
+        graph_strs.append('<g id="g2" type="%s" style="visibility: hidden">\n' % (graph_type))
+        g2_str = get_graph_as_svg_str_for_animation(secondaryGraphDisplay, g2_x_add, g2_y_add, file, idPrefix=id_prefixes[1])
+        graph_strs.append(g2_str)
+        graph_strs.append('</g>\n')
+
+    # Build the Algorithm SVG string
+    source = algorithm.GetSource()
+    tokenize.tokenize(StringIO.StringIO(source.replace('\\', '\\\\')).readline, tokenEater)
+    algowin.CommitStop()
+
+
+    if no_info_file:
+        info_file = ""
+    else:
+        info_file = 'infos/' + fileName[fileName.rindex('/') + 1:]
+
+    if not htmlTitle:
+        htmlTitle = construct_title(fileName)
+
+    if not animationName:
+        animationName = construct_animation_name(fileName)
+        
+    # Merge the animation into the HTML
+    str_vars = {
+        'title': htmlTitle,
+        'animation_name': animationName,
+        'chapter_number': chapter_number or 0,
+        'algo_div': algo_div or '',
+        'chapter_name': CHAPTER_NUM_TO_ABBREV.get(int(chapter_number or 0)) or '',
+        'info_file': info_file,
+        'this_url': fileName[fileName.rindex('/') + 1:],
+        'animation': format_animation(animation),
+        'graph_str': '\n'.join(graph_strs), 
+        'algo_str': '<g id="codelines" style="visibility: hidden">' + ''.join(algo_lines) + "</g>",
+        'g1_x_add': g1_x_add,
+        'g1_y_add': g1_y_add,
+        'g2_x_add': g2_x_add,
+        'g2_y_add': g2_y_add,
+        'g1_init_edge_info': format_init_edge_infos(init_edge_infos[0], id_prefixes[0]) if init_edge_infos else 'null',
+        'g2_init_edge_info': format_init_edge_infos(init_edge_infos[1], id_prefixes[1]) if init_edge_infos and len(init_edge_infos) > 1 else 'null',
+        'g1_init_graph_info': '"%s"' % init_graph_infos[0] if init_graph_infos and init_graph_infos[0] else '""',
+        'g2_init_graph_info': '"%s"' % init_graph_infos[1] if init_graph_infos and init_graph_infos[1] else '""',
+        'g1_init_vertex_info': format_init_vertex_infos(init_vertex_infos[0], id_prefixes[0]) if init_vertex_infos else 'null',
+        'g2_init_vertex_info': format_init_vertex_infos(init_vertex_infos[1], id_prefixes[1]) if init_vertex_infos else 'null',
+        'g1_init_bubbles': format_bubbles(graphDisplay.bubbles),
+        'g2_init_bubbles': format_bubbles(secondaryGraphDisplay.bubbles if secondaryGraphDisplay else None),
+        'g1_init_moats': format_bubbles(graphDisplay.moats),
+        'g2_init_moats': format_bubbles(secondaryGraphDisplay.moats if secondaryGraphDisplay else None),
+    }
+    file.write(animationhead % str_vars)
+    file.close()
+
+    # Export the algorithm info to its own HTML file        
+    ExportAlgoInfo(fileName, algorithm)
+
