@@ -4,14 +4,14 @@
 #       This file is part of Gato (Graph Animation Toolbox) 
 #
 #	file:   GatoTest.py
-#	author: Alexander Schliep (alexander@schliep.org)
+#	author: Alexander Schliep (alexander@schlieplab.org)
 #
-#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and 
-#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
+#       Copyright (C) 2016-2020 Alexander Schliep and
+#	Copyright (C) 1998-2015 Alexander Schliep, Winfried Hochstaettler and 
+#       Copyright (C) 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
 #                                   
-#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de             
+#       Contact: alexander@schlieplab.org             
 #
-#       Information: http://gato.sf.net
 #
 #       This library is free software; you can redistribute it and/or
 #       modify it under the terms of the GNU Library General Public
@@ -29,8 +29,8 @@
 #
 #
 #
-#       This file is version $Revision: 670 $ 
-#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
+#       This file is version $Revision: 806 $ 
+#                       from $Date: 2020-12-08 15:13:58 +0100 (Tue, 08 Dec 2020) $
 #             last change by $Author: schliep $.
 #
 ################################################################################
@@ -143,10 +143,10 @@ instance = {
     '04-LPDuality/PD_Kruskal3.cat'
     ],
     '05-ShortestPaths/BellmanFord.alg':[
-    '05-ShortestPaths/BellmanFordWC.cat'
+    '05-ShortestPaths/BellmanFordWC.cat', 
     ],
     '05-ShortestPaths/Dijkstra.alg':[
-    '05-ShortestPaths/DijkstraRelabel.cat'
+    '05-ShortestPaths/DijkstraRelabel.cat', 
     ],
     '05-ShortestPaths/DijkstraPQ.alg':[
     '05-ShortestPaths/DijkstraRelabel.cat'
@@ -172,7 +172,7 @@ instance = {
     '07-MinimumCostFlows/CostScaling.alg':[
     '07-MinimumCostFlows/MCF4to1B.cat'
     ],
-    '07-MinimumCostFlows/NegativeCircuit.alg':[
+    '07-MinimumCostFlows/NegativeCycleCanceling.alg':[
     '07-MinimumCostFlows/MCFCycle.cat'
     ],
     '07-MinimumCostFlows/SuccessiveShortestPath.alg':[
@@ -199,9 +199,10 @@ svg_instance = [
         'chapter_directory': '02-GraphsNetworks',
         'chapter_number': 2,
         'title': 'Basics, Notation and Data Structures',
+        'shortened_title': 'Basics',
         'algorithms': [
             {
-                'title': 'BFS-components',
+                'title': 'BFS-Components',
                 'file': 'BFS-components.alg',
                 'description': 'Finds connected components in a graph using iterated ' \
                 'Breadth-First-Search (BFS).',
@@ -214,7 +215,7 @@ svg_instance = [
                 'title': 'DFS Recursive',
                 'file': 'DFS-Recursive.alg',
                 'description': 'A recursive implementation of Depth-First-Search (DFS), ' \
-                'which computes a DFS labeling',
+                'which computes a DFS labeling.',
                 'graphs': [
                     '02-GraphsNetworks/BFS.cat', '02-GraphsNetworks/3Components.cat'
                 ]
@@ -223,7 +224,7 @@ svg_instance = [
                 'title': 'Iterative BFS',
                 'file': 'BFS.alg',
                 'description': 'An iterative implementation of Breadth-First-Search ' \
-                '(BFS), which computes a BFS labeling',
+                '(BFS), which computes a BFS labeling.',
                 'graphs': [
                     '02-GraphsNetworks/BFS.cat', '02-GraphsNetworks/3Components.cat'
                 ]
@@ -232,7 +233,7 @@ svg_instance = [
                 'title': 'BFS to DFS',
                 'file': 'BFStoDFS.alg',
                 'description': 'Converting the iterative BFS implementation to an iterative ' \
-                'DFS implementation by exchanging the Queue by a Stack.',
+                'DFS implementation by exchanging the Queue for a Stack.',
                 'graphs': [
                     '02-GraphsNetworks/BFS.cat', '06-MaximalFlows/FordFulkerson5.cat'
                 ]
@@ -241,7 +242,7 @@ svg_instance = [
                 'title': 'Iterative DFS',
                 'file': 'DFS.alg',
                 'description': 'An iterative implementation of Depth-First-Search (DFS), which ' \
-                'computes a DFS labeling',
+                'computes a DFS labeling.',
                 'graphs': [
                     '02-GraphsNetworks/BFS.cat', '02-GraphsNetworks/3Components.cat'
                 ]
@@ -252,6 +253,7 @@ svg_instance = [
         'chapter_directory': '03-MinimalSpanningTrees',
         'chapter_number': 3,
         'title': 'Minimum Spanning Trees',
+        'shortened_title': 'Minimum Spanning Trees',
         'algorithms': [
             {   
                 'title': 'Kruskal\'s Algorithm',
@@ -267,7 +269,7 @@ svg_instance = [
             {
                 'title': 'Kruskal\'s Algorithm using FindCircuit',
                 'file': 'KruskalFindCircuit.alg',
-                'description': 'A more detailled, but naive implementation of Kruskal\'s '\
+                'description': 'A more detailed, but naive implementation of Kruskal\'s ' \
                 'algorithm, which checks for each edge to be added whether it completes a circuit.',
                 'graphs': [
                     '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
@@ -312,7 +314,7 @@ svg_instance = [
             {
                 'title': 'Matroid Dual of Kruskal\'s Algorithm',
                 'file': 'MatroidDualKruskal.alg',
-                'description': 'An implementation of a matroid dual of Kruskal\'s Algorithm. ', 
+                'description': 'An implementation of a matroid dual of Kruskal\'s Algorithm.', 
                 'graphs': [
                     '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
                     '03-MinimalSpanningTrees/Kruskal3.cat',
@@ -325,6 +327,7 @@ svg_instance = [
         'chapter_directory': '04-LPDuality',
         'chapter_number': 4,
         'title': 'Linear Programming Duality',
+        'shortened_title': 'Linear Programming Duality',
         'algorithms': [
             {
                 'title': 'Primal Dual of Kruskal\'s Algorithm',
@@ -342,9 +345,10 @@ svg_instance = [
         'chapter_directory': '05-ShortestPaths',
         'chapter_number': 5,
         'title': 'Shortest Paths',
+        'shortened_title': 'Shortest Paths',
         'algorithms': [
             {
-                'title': 'Dijkstra\'s algorithm',
+                'title': 'Dijkstra\'s Algorithm',
                 'file': 'Dijkstra.alg',
                 'description': 'Dijkstra\'s algorithm for finding a shortest path tree in ' \
                 'a graph. This is an example of a label setting algorithm.',
@@ -352,14 +356,14 @@ svg_instance = [
                     '05-ShortestPaths/ShortestPathsUndirected02.cat',
                     '05-ShortestPaths/ShortestPathsUndirected03.cat',
                     '05-ShortestPaths/7x7grid.cat',
-                    '05-ShortestPaths/NegArc.cat',
-                    '05-ShortestPaths/NegCircuit.cat'                   
+                    '05-ShortestPaths/NegCircuit.cat', 
+                    '05-ShortestPaths/NegArc.cat'
                 ]
             },
             {
-                'title': 'Dijkstra\'s algorithm using a Priority Queue',
+                'title': 'Dijkstra\'s Algorithm using a Priority Queue',
                 'file': 'DijkstraPQ.alg',
-                'description': 'Dijkstra\'s algorithm for finding a shortest path tree in a' \
+                'description': 'Dijkstra\'s algorithm for finding a shortest path tree in a ' \
                 'graph implemented using a Priority Queue.',
                 'graphs': [
                     '05-ShortestPaths/ShortestPathsUndirected02.cat',
@@ -386,7 +390,7 @@ svg_instance = [
                 'description': 'A variant of Dijkstra\'s algorithm for finding a shortest s-t ' \
                 'path in an Euclidean graph (edge weights correspond to Euclidean distance ' \
                 'between vertices). Terminates early, as soon as a shortest s-t path has been ' \
-                'found and only visits a small part of the graph. Similar ideas are used in '\
+                'found and only visits a small part of the graph. Similar ideas are used in ' \
                 'A*-type algorithms.',
                 'graphs': [
                     '05-ShortestPaths/ShortestPathsUndirected03.cat',
@@ -410,18 +414,17 @@ svg_instance = [
             {
                 'title': 'Finding Negative Circuits',
                 'file': 'NegativeCircuits.alg',
-                'description': 'Most shortest path algorithms require that graphs do not have '\
+                'description': 'Most shortest path algorithms require that graphs do not have ' \
                     'negative circuits. This algorithms detects negative circuits in a graph.',
                 'graphs': [
                     '05-ShortestPaths/NegCircuit.cat',
                     '05-ShortestPaths/NegCircuit2.cat'
                 ]
             },
-            # Could use highlighting of paths on this
             {
                 'title': 'Find Path from Two Sources',
                 'file': 'TwoSources.alg',
-                'description': 'Finding a path can be accelerated by searching from start and' \
+                'description': 'Finding a path can be accelerated by searching from start and ' \
                 'destination simultaneously.',
                 'graphs': [
                     '05-ShortestPaths/ShortestPathsUndirected06.cat'
@@ -432,10 +435,11 @@ svg_instance = [
     {
         'chapter_directory': '06-MaximalFlows',
         'chapter_number': 6,
-        'title': 'Maximal flows in capacitated networks',
+        'title': 'Maximal Flows',
+        'shortened_title': 'Maximal Flows',
         'algorithms': [
             {
-                'title': 'The Ford Fulkerson algorithm',
+                'title': 'The Ford-Fulkerson Algorithm',
                 'file': 'FordFulkerson.alg',
                 'description': 'The Ford-Fulkerson algorithm finds a maximal flow in a ' \
                 'capacitated network by successive shortest path applications.',
@@ -454,7 +458,7 @@ svg_instance = [
                 'file': 'PreflowPush.alg',
                 'description': 'The Preflow Push algorithm due to Goldberg and Tarjan ' \
                 'computes a maximal flow by saturating a network and correcting possible ' \
-                'excesses later',
+                'excesses later.',
                 'graphs': [
                     '06-MaximalFlows/PreflowPush1.cat',
                     '06-MaximalFlows/PreflowPush6.cat',
@@ -471,12 +475,13 @@ svg_instance = [
     {
         'chapter_directory': '07-MinimumCostFlows',
         'chapter_number': 7,
-        'title': 'Minimum-cost Flows',
+        'title': 'Minimum-Cost Flows',
+        'shortened_title': 'Minimum-Cost Flows',
         'algorithms': [
             {
-                'title': 'Negative Circuits',
-                'file': 'NegativeCircuit.alg',
-                'description': 'The Negative Circuits algorithms establishes a maximal flow ' \
+                'title': 'Negative Cycle Canceling',
+                'file': 'NegativeCycleCanceling.alg',
+                'description': 'The Negative Cycle Canceling algorithm establishes a maximal flow ' \
                 'and reduces costs by finding negative circuits.',
                 'graphs': [
                     '07-MinimumCostFlows/MCF4to1A.cat', '07-MinimumCostFlows/MCFCycle.cat'
@@ -485,8 +490,8 @@ svg_instance = [
             {
                 'title': 'Successive Shortest Paths',
                 'file': 'SuccessiveShortestPath.alg',
-                'description': 'This algorithm finds a minimum cost flow by  repeatedly ' \
-                ' finding shortest, augmenting paths ',
+                'description': 'This algorithm finds a minimum cost flow by repeatedly ' \
+                'finding shortest, augmenting paths.',
                 'graphs': [
                     '07-MinimumCostFlows/MCF4to1A.cat', '07-MinimumCostFlows/MCFCycle.cat'
                 ]
@@ -495,7 +500,7 @@ svg_instance = [
                 'title': 'Cost Scaling',
                 'file': 'CostScaling.alg',
                 'description': 'The cost scaling algorithm directly operates on the bits ' \
-                'of the cost function',
+                'of the cost function.',
                 'graphs': [
                     '07-MinimumCostFlows/MCF4to1A.cat', '07-MinimumCostFlows/MCFCycle.cat'
                 ]
@@ -507,13 +512,15 @@ svg_instance = [
         'chapter_directory': '08-Matching',
         'chapter_number': 8,
         'title': 'Matching',
+        'shortened_title': 'Matching',
         'algorithms': [
             {
                 'title': 'Bipartite',
                 'file': 'Bipartite.alg',
-                'description': 'This algorithm finds a maximal cardinality matching' \
-                'in a bipartite graph. In other words it maximes the number of matched ' \
-                'vertices',
+                'svg_file': 'Bipartite_app.alg',
+                'description': 'This algorithm finds a maximal cardinality matching ' \
+                'in a bipartite graph. In other words it maximizes the number of matched ' \
+                'vertices.',
                 'graphs': [
                     '08-Matching/Bi003.cat', '08-Matching/Bi006.cat', '08-Matching/Bi008.cat',
                     '08-Matching/Bi009.cat', '08-Matching/Bi010.cat'
@@ -522,6 +529,7 @@ svg_instance = [
             {
                 'title': 'Cardinality Matching',
                 'file': 'CardinalityMatching.alg',
+                'svg_file': 'CardinalityMatching_app.alg',
                 'description': 'This algorithm finds a maximal cardinality matching in graphs ' \
                 'which are not necessarily bipartite.',
                 'graphs': [
@@ -536,12 +544,13 @@ svg_instance = [
         'chapter_directory': '09-WeightedMatching',
         'chapter_number': 9,
         'title': 'Weighted Matching',
+        'shortened_title': 'Weighted Matching',
         'algorithms': [
             {
                 'title': 'Weighted Matching',
                 'file': 'WeightedMatching.alg',
                 'description': 'This algorithm considers vertices as points in 2-dimensional ' \
-                'space and implicitely assumes the existence of all edges in the complete ' \
+                'space and implicitly assumes the existence of all edges in the complete ' \
                 'graph. Furthermore, edge weights are Euclidean. It finds a matching of minimal ' \
                 'weight. The matching is perfect if the number of vertices is even', 
                 'graphs': [
@@ -553,6 +562,70 @@ svg_instance = [
     }
 ]
 
+##svg_instance = [
+##    {
+##        'chapter_directory': '03-MinimalSpanningTrees',
+##        'chapter_number': 3,
+##        'title': 'Minimum Spanning Trees',
+##        'shortened_title': 'Minimum Spanning Trees',
+##        'algorithms': [
+##            {
+##                'title': 'Kruskal\'s Algorithm using FindCircuit',
+##                'file': 'KruskalFindCircuit.alg',
+##                'description': 'A more detailed, but naive implementation of Kruskal\'s ' \
+##                'algorithm, which checks for each edge to be added whether it completes a circuit.',
+##                'graphs': [
+##                    '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat'
+##                ]
+##            }
+##        ]
+##    }
+##]
+
+alphabetic_algorithms = [
+    {'algo_div_id': 'BellmanFord', 'title': 'Bellman Ford'}, 
+
+    {'algo_div_id': 'Bipartite', 'title': 'Bipartite'}, 
+
+    {'algo_div_id': 'IterativeBFS', 'title': 'Iterative BFS'}, 
+    {'algo_div_id': 'BFSComponents', 'title': 'BFS-Components'}, 
+    {'algo_div_id': 'BFStoDFS', 'title': 'BFS to DFS'}, 
+
+    {'algo_div_id': 'CardinalityMatching', 'title': 'Cardinality Matching'}, 
+
+    {'algo_div_id': 'CostScaling', 'title': 'Cost Scaling'}, 
+
+    {'algo_div_id': 'DFSRecursive', 'title': 'DFS Recursive'}, 
+    {'algo_div_id': 'IterativeDFS', 'title': 'Iterative DFS'}, 
+
+    {'algo_div_id': "Dijkstra'sAlgorithm", 'title': "Dijkstra's Algorithm"}, 
+    {'algo_div_id': "Dijkstra'sAlgorithmusingaPriorityQueue", 'title': "Dijkstra's Algorithm using a Priority Queue"}, 
+
+    {'algo_div_id': 'FindingNegativeCircuits', 'title': 'Finding Negative Circuits'},
+
+    {'algo_div_id': 'FindPath', 'title': 'Find Path'}, 
+    {'algo_div_id': 'FindPathfromTwoSources', 'title': 'Find Path from Two Sources'}, 
+    {'algo_div_id': 'FindPathinanEuclideanGraph', 'title': 'Find Path in an Euclidean Graph'}, 
+
+    {'algo_div_id': 'TheFordFulkersonAlgorithm', 'title': 'The Ford-Fulkerson Algorithm'}, 
+
+    {'algo_div_id': "Kruskal'sAlgorithm", 'title': "Kruskal's Algorithm"}, 
+    {'algo_div_id': "Kruskal'sAlgorithmusingFindCircuit", 'title': "Kruskal's Algorithm using FindCircuit"}, 
+    {'algo_div_id': "EfficientKruskal'sAlgorithm", 'title': "Efficient Kruskal's Algorithm"}, 
+    {'algo_div_id': "InefficientKruskal'sAlgorithm", 'title': "Inefficient Kruskal's Algorithm"}, 
+    {'algo_div_id': "MatroidDualofKruskal'sAlgorithm", 'title': "Matroid Dual of Kruskal's Algorithm"}, 
+    {'algo_div_id': "PrimalDualofKruskal'sAlgorithm", 'title': "Primal Dual of Kruskal's Algorithm"}, 
+
+    {'algo_div_id': 'NegativeCycleCanceling', 'title': 'Negative Cycle Canceling'}, 
+
+    {'algo_div_id': 'PreflowPush', 'title': 'Preflow Push'}, 
+
+    {'algo_div_id': "Prim'sAlgorithm", 'title': "Prim's Algorithm"}, 
+
+    {'algo_div_id': 'SuccessiveShortestPaths', 'title': 'Successive Shortest Paths'}, 
+
+    {'algo_div_id': 'WeightedMatching', 'title': 'Weighted Matching'}
+]
 
 #
 # Dictionary of graph name substitutes(e.g. if a graph name is too long we can map it to a different name here, and the sub will be used in the index page)
@@ -568,7 +641,7 @@ friendly_graph_names = {
 
 # Keys: e.g. '09-WeightedMatching/k4.cat', values: graph description
 graph_descriptions = {
-    '02-GraphsNetworks/BFS.cat': 'BFS Graph Description'
+    #'02-GraphsNetworks/BFS.cat': 'BFS Graph Description'
 }
 
 def create_svg_index_page(graph_pngs):
@@ -592,7 +665,8 @@ def create_svg_index_page(graph_pngs):
                 'svg_instance': svg_instance, 
                 'graph_pngs': graph_pngs, 
                 'graph_descriptions': graph_descriptions,
-                'svg_locations': svg_locations
+                'svg_locations': svg_locations,
+                'alphabetic_algorithms': alphabetic_algorithms
              }))
 
 def should_generate_animation(algo_file_name, graph_file_name, svg_file_name):
@@ -608,8 +682,9 @@ def should_generate_animation(algo_file_name, graph_file_name, svg_file_name):
     pro_age = os.path.getmtime(os.path.join(catbox_path, algo_pro_file_name))
     algo_age = os.path.getmtime(algo_file_name)
     webgato_js_age = os.path.getmtime('WebGatoJS.py')
+    this_age = os.path.getmtime('GatoTest.py')
 
-    if svg_age > max(graph_age, pro_age, algo_age, webgato_js_age):
+    if svg_age > max(graph_age, pro_age, algo_age, webgato_js_age, this_age):
         return False
     return True
 
@@ -766,19 +841,24 @@ if __name__ == '__main__':
                     log.info("=== TEST === "+algo['file']+" === "+graph_file+" ===")
                     print "=== TEST === "+algo['file']+" === "+graph_file+" ==="
                     graph_name = os.path.splitext(os.path.basename(graph_file))[0]
-                    png_file_name = 'svgs/img/%s.png' % (graph_name)
+                    png_file_name = 'svgs/img/%s--%s.png' % (os.path.splitext(algo['file'])[0], graph_name)
                     svg_file_name = 'svgs/%s--%s.html' % (os.path.splitext(algo['file'])[0], graph_name)
-                    algo_location = testPath + chapter_dict['chapter_directory'] + '/' + algo['file']
-                    if should_generate_animation(algo_location, testPath+graph_file, svg_file_name) or force_animation:
+                    algo_location = testPath + chapter_dict['chapter_directory'] + '/' + algo.get('svg_file', algo['file'])
+
+                    generate_anim = should_generate_animation(algo_location, testPath+graph_file, svg_file_name)
+                    generate_png = should_generate_png(algo_location, testPath+graph_file, png_file_name) or force_animation
+                    if generate_anim or generate_png or force_animation:
                         app.OpenAlgorithm(algo_location)
+                        app.OpenGraph(testPath + graph_file)
                         g.Interactive = 0 # This is set to 0 above.  Do we need to do it here as well?
                         g.GeneratingSVG = 1
                         app.algorithm.ClearBreakpoints()
+                        app.algorithm.DB.alwaysTrace = 1 # Capture activeLine in sub-routines
                         app.update_idletasks()
                         app.update()
-                        app.OpenGraph(testPath + graph_file)
                         app.update_idletasks()
                         app.update()
+                        # coordinate_diff = app.GetSVGCoordinateDiff()
                         # Run it ...
                         app.after_idle(app.CmdContinue) # after idle needed since CmdStart
                         # does not return
@@ -786,21 +866,27 @@ if __name__ == '__main__':
                         app.update_idletasks()
 
                         # Generate the SVG
-                        app.ExportSVGAnimation(svg_file_name, chapter_number=chapter_dict['chapter_number'], algo_div=algo['title'].replace(' ', '').replace('-',''))
+                        app.ExportSVGAnimation(svg_file_name, chapter_number=chapter_dict['chapter_number'], 
+                            algo_div=algo['title'].replace(' ', '').replace('-',''),
+                            chapter_name=chapter_dict['title'])
+                    else:
+                        print "Animation already generated, skipping."
 
-                    index_graph_name = friendly_graph_names.get(graph_name, graph_name)
-                    if should_generate_png(algo_location, testPath+graph_file, png_file_name):
+                    if generate_png:
                         # Generate the PNG
-                        if graph_file not in graph_pngs and has_png_libs:
+                        index_page_graph_name = friendly_graph_names.get(graph_name, graph_name)
+                        if png_file_name not in graph_pngs and has_png_libs:
+                            # png_dimensions = app.ExportSVG(png_file_name, write_to_png=True, start_graph_coord_diff=coordinate_diff)
                             png_dimensions = app.ExportSVG(png_file_name, write_to_png=True)
                             path_from_index = '/'.join(png_file_name.split('/')[1:])
-                            graph_pngs[graph_file] = {'file': path_from_index, 'name': index_graph_name, 'width': png_dimensions['width'], 'height': png_dimensions['height']}
-                    else:
-                        from PIL import Image
-                        img = Image.open(png_file_name)
-                        graph_pngs[graph_file] = {'file': '/'.join(png_file_name.split('/')[1:]), 'name': index_graph_name, 'width': img.size[0], 'height': img.size[1]}
+                            graph_pngs[algo['title'] + graph_file] = {'file': path_from_index, 'name': index_page_graph_name, 'width': png_dimensions['width'], 'height': png_dimensions['height']}
+                        else:
+                            from PIL import Image
+                            img = Image.open(png_file_name)
+                            graph_pngs[algo['title'] + graph_file] = {'file': '/'.join(png_file_name.split('/')[1:]), 'name': index_page_graph_name, 'width': img.size[0], 'height': img.size[1]}
 
-        if has_png_libs:
+
+        if has_png_libs and graph_pngs:
             create_svg_index_page(graph_pngs)
 
     else:
